/* tableData.h

   Copyright (C) 1999 by David Rose.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   To contact the original author of this program, write to:
           David Rose
           224 W. Dryden St Apt. 418
	   Glendale, CA 91202
	   USA

	   drose@lafn.org
   */

#ifndef TABLEDATA_H
#define TABLEDATA_H

#include "tableDef.h"
#include "dataCell.h"
#include "referenceCount.h"

#include <gtk--sig.h>
#include <gtk--sigcommon.h>
#include <vector>
#include <iostream.h>

class TableDataProxy;

class TableData : public ReferenceCount, public Gtk_Signal_Base {
public:
  TableData(TableID table_id);
  TableData(const TableData &copy);
  TableData &operator = (const TableData &copy);

  virtual ~TableData();

  static TableData *find_table_data(TableID table_id,
				    int primary_key_value);

  TableID get_table_id() const;
  bool has_fields() const;

  void clear();
  bool look_up(int primary_key_value);
  void set_primary_key(int primary_key_value, bool new_data);
  int get_primary_key() const;
  bool is_new_data() const;
  int record_db();
  int record_db(int primary_key_value);

  DataCell &operator [] (int field_id);
  const DataCell operator [] (int field_id) const;

  void set_db_value(int field_id, const DataCell &value);
  const DataCell get_db_value(int field_id) const;

  void output(ostream &out) const;

  class Value {
  public:
    Value();
    Value(const DataCell &current, const DataCell &db);
    bool has_changed() const;

    void set_db_value(const DataCell &value);

    DataCell _current;
    DataCell _db;
  };

private:
  // Mapping field_id to value.
  typedef map<int, Value> Values;
  Values _values;

  TableID _table_id;
  int _primary_key_value;
  bool _new_data;
  const TableDef *_def;

  friend class TableDataProxy;
};

inline ostream &operator << (ostream &out, const TableData &tdata) {
  tdata.output(out);
  return out;
}
    
#endif
